/*
 * This file is part of family-shop-mgr.
 *
 * family-shop-mgr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * family-shop-mgr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with family-shop-mgr.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Unai IRIGOYEN
 * Date: 12/07/2009
 *
 */

#include <QtXml>

#include "ShoppingTreeItem.h"


ShoppingTreeItem::ShoppingTreeItem(QDomNode &node, int row, ShoppingTreeItem *parent)
{
    domNode = node;
    rowNumber = row;
    parentItem = parent;
}

/*******************************************************************/
ShoppingTreeItem::~ShoppingTreeItem()
{
    QHash<int,ShoppingTreeItem*>::iterator it;
    for (it = childItems.begin(); it != childItems.end(); ++it)
        delete it.value();
}

/*******************************************************************/
QDomNode ShoppingTreeItem::node() const
{
    return domNode;
}

/*******************************************************************/
ShoppingTreeItem *ShoppingTreeItem::parent()
{
    return parentItem;
}

/*******************************************************************/
ShoppingTreeItem *ShoppingTreeItem::child(int i)
{
    if (childItems.contains(i))
        return childItems[i];

    QDomNodeList childNodesList = domNode.childNodes();
    QList<QDomNode> childElementsList;
    QDomNode childNode;
    for(int j = 0; !(childNode = childNodesList.at(j)).isNull(); j++)
    {
        if(childNode.isElement())
        {
            if(childNode.toElement().nodeName() == "category" ||
               childNode.toElement().nodeName() == "item")
                childElementsList.append(childNode);
        }
    }
    if (i >= 0 && i < childElementsList.count()) {
        childNode = childElementsList.at(i);
        ShoppingTreeItem *childItem = new ShoppingTreeItem(childNode, i, this);
        childItems[i] = childItem;
        return childItem;
    }
    return 0;
}

/*******************************************************************/
int ShoppingTreeItem::row()
{
    return rowNumber;
}
