/*
 * This file is part of family-shop-mgr.
 *
 * family-shop-mgr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * family-shop-mgr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with family-shop-mgr.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Unai IRIGOYEN
 * Date: 12/07/2009
 *
 */

#ifndef SHOPPINGTREEMODEL_H
#define SHOPPINGTREEMODEL_H

#include <QAbstractItemModel>
#include <QDomDocument>
#include <QModelIndex>
#include <QVariant>

class ShoppingTreeItem;

enum ItemType{
    item,
    category
};

class ShoppingTreeModel : public QAbstractItemModel
{
    Q_OBJECT

public:
    ShoppingTreeModel(QString fileName, QObject *parent = 0);
    ~ShoppingTreeModel();

    QVariant data(const QModelIndex &index, int role) const;
    Qt::ItemFlags flags(const QModelIndex &index) const;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role = Qt::DisplayRole) const;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const;
    QModelIndex parent(const QModelIndex &child) const;
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;
    bool insertRows(int row, int count,
                    const QModelIndex & parent = QModelIndex(),
                    ItemType type = item);
    bool removeRows(int row, int count, const QModelIndex & parent = QModelIndex());
    bool setData(const QModelIndex & index, const QVariant & value, int role = Qt::EditRole);

public slots:
    bool addCategory(QModelIndex & index, QString name = QString());
    bool addItem(QModelIndex & index, QString name = QString());
    bool removeElement(QModelIndex & index);
    void save();

signals:
    void documentChanged();

private:
    QString m_fileName;
    QDomDocument domDocument;
    QDomNode rootItemNode;
    ShoppingTreeItem *rootItem;
};

#endif // SHOPPINGTREEMODEL_H
